#!/bin/bash

# reverb response generator script
# ReverbGen.sh

if [ "${INCDIR}" == "" ] ; then
	INCDIR="/Csound/Include" ;	# include directory
fi

# print_usage <exit status>
# print program usage and exit

function	print_usage ()
{
	echo -e -n "Usage: ReverbGen.sh <scorefile> <outfile> <srate>"
		echo -e " <spatmacro> [options ...]"
	echo -e    "       ReverbGen.sh <conf.file>"
	echo -e    ""
	echo -e    "<scorefile> : score file to set reverb parameters"
	echo -e    "              (${INCDIR}/Parameters.Reverb is an example"
	echo -e    "              file with default settings)"
	echo -e    "<outfile> :   output file"
	echo -e    "<srate> :     sample rate of generated impulse response"
	echo -e -n "<spatmacro> : macro to be used for spatialization,"
		echo -e " available macros are:"
	echo -e    "              MonoStereo, BaseStereo, SpatStereo,"
	echo -e    "              PhseStereo, UHJ_Stereo, and HRTFStereo"
	echo -e    "options:"
	echo -e    "    --play :  play the rendered impulse response"
	echo -e    "    --nocv :  do not convert to convolve format"
	echo -e -n "<conf.file> : read command-line parameters from the"
		echo -e " specified file"
	echo -e -n "              (parameters may be separated by spaces, tabs"
		echo -e " and newlines)"
	echo -e ""

	if [ $# != 0 ] ; then
		exit $1 ;
	else
		exit 0 ;
	fi ;
}

# calc_expr <scale> <expression>
# evaluate math expression, and print to stdout

function	calc_expr ()
{
	echo -e " scale=${1} ; ${2} " | bc -l -q | tr -d " \n\t" ;
}

# check_file <filename>
# exit if "filename" does not exist or is not a file

function	check_file ()
{
	if ( ! ( test -e "${1}" ) ) ; then
		echo -e "ReverbGen.sh: ${1}: not found" ; exit -1 ;
	elif ( ! ( test -f "${1}" ) ) ; then
		echo -e "ReverbGen.sh: ${1}: not a file" ; exit -1 ;
	fi ;
}

# print usage if there are insufficient args

if ( ( [ $# == 0 ] ) || ( [ $# == 2 ] ) || ( [ $# == 3 ] ) ) ; then
	print_usage -1 ;
fi

if [ $# == 1 ] ; then		# options are stored in conf. file
	check_file "${1}"
	exec ReverbGen.sh `cat "${1}" | tr "\n\r\t" "   "` ;
fi

PLAYMODE=""	# default: play disabled, convert output to cv file
CVMODE="X"

SCORENAME="${1}" ; shift		# score file name
OUTFLNAME="${1}" ; shift		# output file name
SAMPLRATE="${1}" ; shift		# sample rate
SPATMACRO="${1}" ; shift		# spatializer macro

while [ $# != 0 ] ; do			# optional args
	case "${1}" in
		("--play")	PLAYMODE="X"	;;	# play output file
		("--nocv")	CVMODE=""	;;	# do not run cvanal
		(*)		echo -e "ReverbGen.sh: ${1}: invalid option"
				exit -1		;;
	esac
	shift ;
done

SRATE=`calc_expr 3 "${SAMPLRATE}"`		# sample rate
KSMPS="2"					# ksmps

# check parameters

case "${SPATMACRO}" in
	("MonoStereo")	DECDMACRO="DECODE_MONO" ;	SPATMODE="0"	;;
	("BaseStereo")	DECDMACRO="DECODE_STEREO1" ;	SPATMODE="1"	;;
	("SpatStereo")	DECDMACRO="DECODE_STEREO2" ;	SPATMODE="4"	;;
	("PhseStereo")	DECDMACRO="DECODE_STEREO3" ;	SPATMODE="3"	;;
	("UHJ_Stereo")	DECDMACRO="DECODE_UHJ" ;	SPATMODE="2"	;;
	("HRTFStereo")	DECDMACRO="DECODE_STEREO2" ;	SPATMODE="4"	;;
	(*)		echo -e "ReverbGen.sh: ${SPATMACRO}: invalid option"
			exit -1						;;
esac

if [ "${SPATMACRO}" == "HRTFStereo" ] ; then	# min, max sample rate
	SR_MIN="44100" ; SR_MAX="44100" ;
else
	SR_MIN="1000" ; SR_MAX="800000" ;
fi
SR_EXP="a=1; if (${SRATE} < ${SR_MIN}) a=0; if (${SRATE} > ${SR_MAX}) a=0; a"
SR_CHK=`calc_expr 0 "${SR_EXP}"`
if [ "${SR_CHK}" != "1" ] ; then
	echo -e "ReverbGen.sh: ${SAMPLRATE}: invalid sample rate"
	exit -1 ;
fi

check_file "${SCORENAME}"			# check score file

TMPORCNAME="/tmp/ReverbGen.tmp.orc.$$"		# temp .orc file name
TMPSCONAME="/tmp/ReverbGen.tmp.sco.$$"		# temp .sco file name
TMPFILENAME="/tmp/ReverbGen.tmp.sf.$$"		# temp output file name

# remove old files

rm -f "${OUTFLNAME}" "${TMPORCNAME}" "${TMPFILENAME}" "${TMPSCONAME}" score.srt

# create orchestra file

cat "${INCDIR}/ReverbGen.orc" |						\
	sed "s/__SRATE/${SRATE}/g" |					\
	sed "s/__KSMPS/${KSMPS}/g" |					\
	sed "s/__MODE/${SPATMODE}/g" |					\
	sed "s/__SPAT1/\$${SPATMACRO}/g" |				\
	sed "s/__DECD1/\$${DECDMACRO}/g"		>  "${TMPORCNAME}"

# score file

echo -e "\nt 0 60\n"					>  "${TMPSCONAME}"
cat "${SCORENAME}"					>> "${TMPSCONAME}"
echo -e "\ni 1 0 1.0 \$Length\n\ne\n"			>> "${TMPSCONAME}"

# render IR file

csound -d -m0 -H0 -f -J -b 65536 -B 65536 -o "${TMPFILENAME}"	\
	"${TMPORCNAME}" "${TMPSCONAME}"

# play output if "--play" is specified

if [ "${PLAYMODE}" != "" ] ; then
	PlaySound "${TMPFILENAME}" -s ;
fi

# convert to convolve format (if enabled)

if [ "${CVMODE}" != "" ] ; then
	csound -U cvanal "${TMPFILENAME}" "${OUTFLNAME}" ;
else
	mv -f "${TMPFILENAME}" "${OUTFLNAME}" ;
fi

# remove tmp files

rm -f "${TMPORCNAME}" "${TMPFILENAME}" "${TMPSCONAME}" score.srt

